@@ -19,13 +19,14 @@ from hypercorn.config import Config
 from hypercorn.asyncio import serve
 from urllib.parse import urlparse
 from time import perf_counter
-import jwt
 import json
 from base64 import b64encode
 from starlette_prometheus import metrics, PrometheusMiddleware
 from delphai_utils.utils import generate_default_message
+from jose.exceptions import JWTError
+from delphai_utils.keycloak import decode_token, PublicKeyFetchError
+
 supported_methods = ['get', 'put', 'post', 'delete', 'patch']
-user_cache = {}
 status_map = {
   StatusCode.OK: 200,
   StatusCode.CANCELLED: 499,
@@ -40,8 +41,8 @@ status_map = {
   StatusCode.FAILED_PRECONDITION: 412,
   StatusCode.ABORTED: 499,
   StatusCode.OUT_OF_RANGE: 416,
-  StatusCode.UNIMPLEMENTED: 404,
-  StatusCode.INTERNAL: 418,
+  StatusCode.UNIMPLEMENTED: 501,
+  StatusCode.INTERNAL: 500,
   StatusCode.UNAVAILABLE: 503,
   StatusCode.DATA_LOSS: 420
 }
@@ -103,23 +104,36 @@ def get_http_handlers(descriptor: FileDescriptor, service_name: str, method: str
     if len(await request.body()) > 0:
       body = await request.json()
     input = {**request.path_params, **request.query_params, **body}
-    try:
-      metadata = {}
-      if 'Authorization' not in request.headers:
-        logging.warn('no authorization header')
-      else:
-        bearer_token = request.headers['Authorization']
-        if 'x-id-token' in request.headers:
-          logging.info('id_token has been provided')
-          id_token = request.headers['x-id-token']
-          user = jwt.decode(id_token, verify=False)
-          user_json = json.dumps(user).encode('ascii')
-          user_b64 = b64encode(user_json).decode('utf-8')
-          metadata['x-delphai-user'] = user_b64
-        elif bearer_token in user_cache:
-          logging.info('bearer_token in user_cache')
-          metadata['x-delphai-user'] = user_cache[bearer_token]
 
+    if 'Authorization' not in request.headers:
+      error_msg = 'No authorization header provided.'
+      logging.error(error_msg)
+      raise HTTPException(401, detail=error_msg)
+    authorization_header = request.headers.get('Authorization')
+    if 'Bearer ' not in authorization_header:
+      error_msg = 'Authorization header has the wrong format.'
+      logging.error(error_msg, authorization_header)
+      raise HTTPException(401, detail=error_msg)
+    _, access_token = authorization_header.split('Bearer ')
+    try:
+      decoded_access_token = await decode_token(access_token)
+    except JWTError as ex:
+      error_msg = f'Error decoding the token: {ex}'
+      logging.error(error_msg)
+      raise HTTPException(401, detail=error_msg)
+    except PublicKeyFetchError as ex:
+      error_msg = f'Error fetching jwk from keycloak: {ex}'
+      logging.error(error_msg)
+      raise HTTPException(502, detail=error_msg)
+    user = {
+      'https://delphai.com/mongo_user_id': decoded_access_token.get('mongo_user_id'),
+      'https://delphai.com/client_id': decoded_access_token.get('mongo_client_id')
+    }
+    user_json = json.dumps(user).encode('ascii')
+    user_b64 = b64encode(user_json).decode('utf-8')
+    metadata = {'x-delphai-user': user_b64}
+
+    try:
       raw_output = await process_grpc_request(descriptor, service_name, method, input, channel, metadata=metadata)
       output = MessageToDict(raw_output, preserving_proto_field_name=True)
       return JSONResponse(output)
